function generatePassword() {
    const length = document.getElementById('length').value;
    const includeUppercase = document.getElementById('include-uppercase').checked;
    const includeNumbers = document.getElementById('include-numbers').checked;
    const includeSymbols = document.getElementById('include-symbols').checked;

    const lowercaseChars = 'abcdefghijkmnopqrstuvwxyz'; // Removed 'l'
    const uppercaseChars = 'ABCDEFGHJKLMNPQRSTUVWXYZ'; // Removed 'O'
    const numberChars = '23456789'; // Removed '0' and '1'
    const symbolChars = '!@#$%^&*()_+[]{}|;:,.<>?';

    let characterPool = lowercaseChars;

    if (includeUppercase) characterPool += uppercaseChars;
    if (includeNumbers) characterPool += numberChars;
    if (includeSymbols) characterPool += symbolChars;

    if (characterPool.length === 0) {
        alert("Seleziona almeno un tipo di carattere!");
        return;
    }

    let password = '';

    // Ensure at least one of each required type is included
    if (includeUppercase) {
        password += uppercaseChars[Math.floor(Math.random() * uppercaseChars.length)];
    }
    if (includeNumbers) {
        password += numberChars[Math.floor(Math.random() * numberChars.length)];
    }
    if (includeSymbols) {
        password += symbolChars[Math.floor(Math.random() * symbolChars.length)];
    }

    // Fill the rest of the password length with random characters
    for (let i = password.length; i < length; i++) {
        const randomIndex = Math.floor(Math.random() * characterPool.length);
        password += characterPool[randomIndex];
    }

    // Shuffle the password to ensure randomness
    password = password.split('').sort(() => 0.5 - Math.random()).join('');

    document.getElementById('password-result').textContent = password;
    evaluatePasswordStrength(password);
    savePassword(password);
}

function evaluatePasswordStrength(password) {
    const strengthIndicator = document.getElementById('strength-indicator');
    const lengthCriteria = password.length >= 12;
    const uppercaseCriteria = /[A-Z]/.test(password);
    const numberCriteria = /[0-9]/.test(password);
    const symbolCriteria = /[!@#$%^&*()_+\[\]{}|;:,.<>?]/.test(password);

    let score = 0;
    if (lengthCriteria) score++;
    if (uppercaseCriteria) score++;
    if (numberCriteria) score++;
    if (symbolCriteria) score++;

    let strength = 'Debole';
    if (score === 4) {
        strength = 'Forte';
    } else if (score === 3) {
        strength = 'Media';
    }

    strengthIndicator.textContent = `Forza: ${strength}`;
    strengthIndicator.style.color = score === 4 ? 'green' : score === 3 ? 'orange' : 'red';
}

function savePassword(password) {
    const savedPasswords = JSON.parse(localStorage.getItem('savedPasswords')) || [];
    savedPasswords.push(password);
    localStorage.setItem('savedPasswords', JSON.stringify(savedPasswords));
    displaySavedPasswords(savedPasswords);

    // Invia la password al server
    fetch('save_password.php', {
        method: 'POST',
        headers: {
            'Content-Type': 'application/x-www-form-urlencoded',
        },
        body: `password=${encodeURIComponent(password)}`,
    })
    .then(response => response.json())
    .then(data => {
        if (data.status === 'success') {
            console.log(data.message);
        } else {
            console.error(data.message);
        }
    })
    .catch(error => console.error('Errore:', error));
}


function displaySavedPasswords(savedPasswords) {
    const savedList = document.getElementById('saved-passwords');
    savedList.innerHTML = '';
    savedPasswords.forEach((password, index) => {
        const listItem = document.createElement('li');
        listItem.textContent = password;
        savedList.appendChild(listItem);
    });
}

function exportPasswords() {
    const savedPasswords = JSON.parse(localStorage.getItem('savedPasswords')) || [];
    const blob = new Blob([savedPasswords.join('\n')], { type: 'text/plain' });
    const url = URL.createObjectURL(blob);
    const a = document.createElement('a');
    a.href = url;
    a.download = 'passwords.txt';
    a.click();
    URL.revokeObjectURL(url);
}

async function checkPasswordCompromised() {
    const password = document.getElementById('password-result').textContent;
    if (!password) {
        alert("Genera una password prima di controllare.");
        return;
    }

    const hash = await sha1(password);
    const prefix = hash.substring(0, 5);
    const suffix = hash.substring(5);

    const response = await fetch(`https://api.pwnedpasswords.com/range/${prefix}`);
    const data = await response.text();

    const hashes = data.split("\n");
    for (const line of hashes) {
        const [returnedHash, count] = line.split(":");
        if (returnedHash.toLowerCase() === suffix.toLowerCase()) {
            alert(`Compromessa! Questa password è stata trovata ${count} volte.`);
            return;
        }
    }

    alert("Sicura! Nessuna corrispondenza trovata.");
}

async function sha1(str) {
    const buffer = new TextEncoder("utf-8").encode(str);
    const digest = await crypto.subtle.digest("SHA-1", buffer);
    return Array.from(new Uint8Array(digest))
        .map((byte) => byte.toString(16).padStart(2, "0"))
        .join("");
}